@@ -61,6 +61,7 @@ that can be `read()`.
 
 import re
 from StringIO import StringIO
+from collections import namedtuple
 
 from genshi import Markup, Stream
 from genshi.core import TEXT, START, END, START_NS, END_NS
@@ -72,7 +73,7 @@ from trac.core import Component, ExtensionPoint, Interface, TracError, \
                       implements
 from trac.resource import Resource
 from trac.util import Ranges, content_disposition
-from trac.util.text import exception_to_unicode, to_utf8, to_unicode
+from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _, tag_
 
 
@@ -81,17 +82,17 @@ __all__ = ['Context', 'Mimeview', 'RenderingContext', 'get_mimetype',
 
 
 class RenderingContext(object):
-    """
-    A rendering context specifies ''how'' the content should be rendered.
+    """A rendering context specifies ''how'' the content should be rendered.
 
     It holds together all the needed contextual information that will be
     needed by individual renderer components.
 
-    To that end, a context keeps track of the Href instance (`.href`) which
-    should be used as a base for building URLs.
+    To that end, a context keeps track of the Href instance
+    (``.href``) which should be used as a base for building URLs.
 
-    It also provides a `PermissionCache`  (`.perm`) which can be used to
-    restrict the output so that only the authorized information is shown.
+    It also provides a `PermissionCache` (``.perm``) which can be used
+    to restrict the output so that only the authorized information is
+    shown.
 
     A rendering context may also be associated to some Trac resource which
     will be used as the implicit reference when rendering relative links
@@ -100,7 +101,7 @@ class RenderingContext(object):
 
     Rendering contexts can be nested, and a new context can be created from
     an existing context using the call syntax. The previous context can be
-    retrieved using the `.parent` attribute.
+    retrieved using the ``.parent`` attribute.
 
     For example, when rendering a wiki text of a wiki page, the context will
     be associated to a resource identifying that wiki page.
@@ -110,6 +111,7 @@ class RenderingContext(object):
     rendering the ticket descriptions.
 
     :since: version 1.0
+
     """
 
     def __init__(self, resource, href=None, perm=None):
@@ -133,8 +135,9 @@ class RenderingContext(object):
 
     @staticmethod
     def from_request(*args, **kwargs):
-        """:deprecated: since 1.0, use `web_context` instead. Will be removed
-                        in release 1.3.1.
+        """
+        :deprecated: since 1.0, use `web_context` instead. Will be removed
+                     in release 1.3.1.
         """
         from trac.web.chrome import web_context
         return web_context(*args, **kwargs)
@@ -525,7 +528,8 @@ class IHTMLPreviewRenderer(Interface):
         """
 
     def render(context, mimetype, content, filename=None, url=None):
-        """Render an XHTML preview of the raw `content` in a RenderingContext.
+        """Render an XHTML preview of the raw ``content`` in a
+        `RenderingContext`.
 
         The `content` might be:
          * a `str` object
@@ -542,6 +546,7 @@ class IHTMLPreviewRenderer(Interface):
         Can return the generated XHTML text as a single string or as an
         iterable that yields strings. In the latter case, the list will
         be considered to correspond to lines of text in the original content.
+
         """
 
 
@@ -634,21 +639,21 @@ class Mimeview(Component):
         """Charset to be used when in doubt.""")
 
     tab_width = IntOption('mimeviewer', 'tab_width', 8,
-        """Displayed tab width in file preview. (''since 0.9'')""")
+        """Displayed tab width in file preview.""")
 
     max_preview_size = IntOption('mimeviewer', 'max_preview_size', 262144,
-        """Maximum file size for HTML preview. (''since 0.9'')""")
+        """Maximum file size for HTML preview.""")
 
     mime_map = ListOption('mimeviewer', 'mime_map',
         'text/x-dylan:dylan, text/x-idl:ice, text/x-ada:ads:adb',
         doc="""List of additional MIME types and keyword mappings.
         Mappings are comma-separated, and for each MIME type,
         there's a colon (":") separated list of associated keywords
-        or file extensions. (''since 0.10'')
+        or file extensions.
         """)
 
     mime_map_patterns = ListOption('mimeviewer', 'mime_map_patterns',
-        'text/plain:README|INSTALL|COPYING.*',
+        'text/plain:README(?!\.rst)|INSTALL(?!\.rst)|COPYING.*',
         doc="""List of additional MIME types associated to filename patterns.
         Mappings are comma-separated, and each mapping consists of a MIME type
         and a Python regexp used for matching filenames, separated by a colon
@@ -657,9 +662,10 @@ class Mimeview(Component):
 
     treat_as_binary = ListOption('mimeviewer', 'treat_as_binary',
         'application/octet-stream, application/pdf, application/postscript, '
-        'application/msword,application/rtf,',
+        'application/msword, application/rtf',
         doc="""Comma-separated list of MIME types that should be treated as
-        binary data. (''since 0.11.5'')""")
+        binary data.
+        """)
 
     def __init__(self):
         self._mime_map = None
@@ -668,16 +674,21 @@ class Mimeview(Component):
     # Public API
 
     def get_supported_conversions(self, mimetype):
-        """Return a list of target MIME types in same form as
-        `IContentConverter.get_supported_conversions()`, but with the converter
-        component appended. Output is ordered from best to worst quality."""
+        """Return a list of target MIME types as instances of the `namedtuple`
+        `MimeConversion`. Output is ordered from best to worst quality.
+
+        The `MimeConversion` `namedtuple` has fields: key, name, extension,
+        in_mimetype, out_mimetype, quality, converter.
+        """
+        fields = ('key', 'name', 'extension', 'in_mimetype',
+                  'out_mimetype', 'quality', 'converter')
+        _MimeConversion = namedtuple('MimeConversion', fields)
         converters = []
-        for converter in self.converters:
-            conversions = converter.get_supported_conversions() or []
-            for k, n, e, im, om, q in conversions:
+        for c in self.converters:
+            for k, n, e, im, om, q in c.get_supported_conversions() or []:
                 if im == mimetype and q > 0:
-                    converters.append((k, n, e, im, om, q, converter))
-        converters = sorted(converters, key=lambda i: i[-2], reverse=True)
+                    converters.append(_MimeConversion(k, n, e, im, om, q, c))
+        converters = sorted(converters, key=lambda i: i.quality, reverse=True)
         return converters
 
     def convert_content(self, req, mimetype, content, key, filename=None,
@@ -700,17 +711,18 @@ class Mimeview(Component):
             mimetype = full_mimetype = 'text/plain'  # fallback if not binary
 
         # Choose best converter
-        candidates = list(self.get_supported_conversions(mimetype) or [])
-        candidates = [c for c in candidates if key in (c[0], c[4])]
+        candidates = [c for c in self.get_supported_conversions(mimetype)
+                        if key in (c.key, c.out_mimetype)]
         if not candidates:
             raise TracError(
                 _("No available MIME conversions from %(old)s to %(new)s",
                   old=mimetype, new=key))
 
         # First successful conversion wins
-        for ck, name, ext, input_mimettype, output_mimetype, quality, \
-                converter in candidates:
-            output = converter.convert_content(req, mimetype, content, ck)
+        for conversion in candidates:
+            output = conversion.converter.convert_content(req, mimetype,
+                                                          content,
+                                                          conversion.key)
             if output:
                 content, content_type = output
                 if iterable:
@@ -719,7 +731,7 @@ class Mimeview(Component):
                 else:
                     if not isinstance(content, basestring):
                         content = ''.join(content)
-                return content, content_type, ext
+                return content, content_type, conversion.extension
         raise TracError(
             _("No available MIME conversions from %(old)s to %(new)s",
               old=mimetype, new=key))
@@ -817,17 +829,19 @@ class Mimeview(Component):
 
                 # Render content as source code
                 if annotations:
-                    m = context.req.args.get('marks') if context.req else None
-                    return self._render_source(context, result, annotations,
-                                               m and Ranges(m))
+                    marks = context.req.args.get('marks') if context.req \
+                            else None
+                    if marks:
+                        context.set_hints(marks=marks)
+                    return self._render_source(context, result, annotations)
                 else:
                     if isinstance(result, list):
                         result = Markup('\n').join(result)
                     return tag.div(class_='code')(tag.pre(result)).generate()
 
-            except Exception, e:
-                self.log.warning('HTML preview using %s failed: %s',
-                                 renderer.__class__.__name__,
+            except Exception as e:
+                self.log.warning('HTML preview using %s with %r failed: %s',
+                                 renderer.__class__.__name__, context,
                                  exception_to_unicode(e, traceback=True))
                 if context.req and not context.get_hint('disable_warnings'):
                     from trac.web.chrome import add_warning
@@ -836,7 +850,7 @@ class Mimeview(Component):
                           renderer=renderer.__class__.__name__,
                           err=exception_to_unicode(e)))
 
-    def _render_source(self, context, stream, annotations, marks=None):
+    def _render_source(self, context, stream, annotations):
         from trac.web.chrome import add_warning
         annotators, labels, titles = {}, {}, {}
         for annotator in self.annotators:
@@ -861,7 +875,7 @@ class Mimeview(Component):
             annotator = annotators[a]
             try:
                 data = (annotator, annotator.get_annotation_data(context))
-            except TracError, e:
+            except TracError as e:
                 self.log.warning("Can't use annotator '%s': %s", a, e)
                 add_warning(context.req, tag.strong(
                     tag_("Can't use %(annotator)s annotator: %(error)s",
@@ -879,8 +893,6 @@ class Mimeview(Component):
         def _body_rows():
             for idx, line in enumerate(_group_lines(stream)):
                 row = tag.tr()
-                if marks and idx + 1 in marks:
-                    row(class_='hilite')
                 for annotator, data in annotator_datas:
                     if annotator:
                         annotator.annotate_row(context, row, idx+1, line, data)
@@ -894,11 +906,6 @@ class Mimeview(Component):
             tag.tbody(_body_rows())
         )
 
-    def get_max_preview_size(self):
-        """:deprecated: since 0.10, use `max_preview_size` attribute directly.
-        """
-        return self.max_preview_size
-
     def get_charset(self, content='', mimetype=None):
         """Infer the character encoding from the `content` or the `mimetype`.
 
@@ -968,7 +975,7 @@ class Mimeview(Component):
                     mimetype, regexp = mapping.split(':', 1)
                 try:
                     self._mime_map_patterns[mimetype] = re.compile(regexp)
-                except re.error, e:
+                except re.error as e:
                     self.log.warning("mime_map_patterns contains invalid "
                                      "regexp '%s' for mimetype '%s' (%s)",
                                      regexp, mimetype, exception_to_unicode(e))
@@ -986,13 +993,6 @@ class Mimeview(Component):
             return True
         return False
 
-    def to_utf8(self, content, mimetype=None):
-        """Convert an encoded `content` to utf-8.
-
-        :deprecated: since 0.10, you should use `unicode` strings only.
-        """
-        return to_utf8(content, self.get_charset(content, mimetype))
-
     def to_unicode(self, content, mimetype=None, charset=None):
         """Convert `content` (an encoded `str` object) to an `unicode` object.
 
@@ -1159,12 +1159,22 @@ class LineNumberAnnotator(Component):
         return 'lineno', _('Line'), _('Line numbers')
 
     def get_annotation_data(self, context):
-        return None
+        try:
+            marks = Ranges(context.get_hint('marks'))
+        except ValueError:
+            marks = None
+        return {
+            'id': context.get_hint('id', '') + 'L%s',
+            'marks': marks,
+            'offset': context.get_hint('lineno', 1) - 1
+        }
 
     def annotate_row(self, context, row, lineno, line, data):
-        row.append(tag.th(id='L%s' % lineno)(
-            tag.a(lineno, href='#L%s' % lineno)
-        ))
+        lineno += data['offset']
+        id = data['id'] % lineno
+        if data['marks'] and lineno in data['marks']:
+            row(class_='hilite')
+        row.append(tag.th(id=id)(tag.a(lineno, href='#' + id)))
 
 
 # -- Default renderers
