@@ -23,6 +23,7 @@ import inspect
 import os
 import shutil
 import sys
+import time
 import types
 import unittest
 import StringIO
@@ -386,6 +387,7 @@ class EnvironmentStub(Environment):
         # -- logging
         from trac.log import logger_handler_factory
         self.log, self._log_handler = logger_handler_factory('test')
+        self.log.addHandler(self._log_handler)
 
         # -- database
         self.dburi = get_dburi()
@@ -484,7 +486,7 @@ class EnvironmentStub(Environment):
         """
         self.reset_db()
         self.shutdown() # really closes the db connections
-        shutil.rmtree(self.path)
+        rmtree(self.env.path)
         if self._old_registry is not None:
             self.restore_component_registry()
 
@@ -536,6 +538,34 @@ def locate(fn):
     return None
 
 
+def rmtree(path):
+    import errno
+    def onerror(function, path, excinfo, retry=1):
+        # `os.remove` fails for a readonly file on Windows.
+        # Then, it attempts to be writable and remove.
+        if function != os.remove:
+            raise
+        e = excinfo[1]
+        if isinstance(e, OSError) and e.errno == errno.EACCES:
+            mode = os.stat(path).st_mode
+            os.chmod(path, mode | 0666)
+            try:
+                function(path)
+            except Exception:
+                # print "%d: %s %o" % (retry, path, os.stat(path).st_mode)
+                if retry > 10:
+                    raise
+                time.sleep(0.1)
+                onerror(function, path, excinfo, retry + 1)
+        else:
+            raise
+    if os.name == 'nt' and isinstance(path, str):
+        # Use unicode characters in order to allow non-ansi characters
+        # on Windows.
+        path = unicode(path, sys.getfilesystemencoding())
+    shutil.rmtree(path, onerror=onerror)
+
+
 INCLUDE_FUNCTIONAL_TESTS = True
 
 
