@@ -17,9 +17,9 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from datetime import datetime, timedelta
 import pkg_resources
 import re
+from datetime import datetime, timedelta
 
 from genshi.builder import tag
 
@@ -27,16 +27,16 @@ from trac.config import IntOption, BoolOption
 from trac.core import *
 from trac.perm import IPermissionRequestor
 from trac.timeline.api import ITimelineEventProvider
-from trac.util import as_int
-from trac.util.datefmt import format_date, format_datetime, format_time, \
-                              parse_date, to_utimestamp, to_datetime, utc, \
-                              pretty_timedelta, user_time
+from trac.util.datefmt import (datetime_now, format_date, format_datetime,
+                               format_time, localtz, parse_date,
+                               pretty_timedelta, utc, to_datetime,
+                               to_utimestamp, user_time)
 from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _, tag_
 from trac.web import IRequestHandler, IRequestFilter
-from trac.web.chrome import (Chrome, INavigationContributor, ITemplateProvider,
-                             add_link, add_stylesheet, add_warning, auth_link,
-                             prevnext_nav, web_context)
+from trac.web.chrome import (Chrome, INavigationContributor,
+                             ITemplateProvider, add_link, add_stylesheet,
+                             add_warning, auth_link, prevnext_nav, web_context)
 from trac.wiki.api import IWikiSyntaxProvider
 from trac.wiki.formatter import concat_path_query_fragment, \
                                 split_url_into_path_query_fragment
@@ -51,11 +51,12 @@ class TimelineModule(Component):
 
     default_daysback = IntOption('timeline', 'default_daysback', 30,
         """Default number of days displayed in the Timeline, in days.
-        (''since 0.9.'')""")
+        """)
 
     max_daysback = IntOption('timeline', 'max_daysback', 90,
         """Maximum number of days (-1 for unlimited) displayable in the
-        Timeline. (''since 0.11'')""")
+        Timeline.
+        """)
 
     abbreviated_messages = BoolOption('timeline', 'abbreviated_messages',
                                       True,
@@ -63,7 +64,7 @@ class TimelineModule(Component):
 
         This only affects the default rendering, and can be overriden by
         specific event providers, see their own documentation.
-        (''Since 0.11'')""")
+        """)
 
     _authors_pattern = re.compile(r'(-)?(?:"([^"]*)"|\'([^\']*)\'|([^\s]+))')
 
@@ -73,7 +74,7 @@ class TimelineModule(Component):
         return 'timeline'
 
     def get_navigation_items(self, req):
-        if 'TIMELINE_VIEW' in req.perm:
+        if 'TIMELINE_VIEW' in req.perm('timeline'):
             yield ('mainnav', 'timeline',
                    tag.a(_("Timeline"), href=req.href.timeline(), accesskey=2))
 
@@ -88,36 +89,36 @@ class TimelineModule(Component):
         return req.path_info == '/timeline'
 
     def process_request(self, req):
-        req.perm.assert_permission('TIMELINE_VIEW')
+        req.perm('timeline').require('TIMELINE_VIEW')
 
-        format = req.args.get('format')
-        maxrows = int(req.args.get('max', 50 if format == 'rss' else 0))
-        lastvisit = int(req.session.get('timeline.lastvisit', '0'))
+        format = req.args.getfirst('format')
+        maxrows = req.args.getint('max', 50 if format == 'rss' else 0)
+        lastvisit = req.session.as_int('timeline.lastvisit', 0)
 
         # indication of new events is unchanged when form is updated by user
-        revisit = any(a in req.args for a in ['update', 'from', 'daysback',
-                                              'author'])
+        revisit = any(a in req.args
+                      for a in ['update', 'from', 'daysback', 'author'])
         if revisit:
-            lastvisit = int(req.session.get('timeline.nextlastvisit',
-                                            lastvisit))
+            lastvisit = req.session.as_int('timeline.nextlastvisit',
+                                           lastvisit)
 
         # Parse the from date and adjust the timestamp to the last second of
         # the day
-        fromdate = today = datetime.now(req.tz)
+        fromdate = today = datetime_now(req.tz)
         yesterday = to_datetime(today.replace(tzinfo=None) - timedelta(days=1),
                                 req.tz)
         precisedate = precision = None
         if 'from' in req.args:
             # Acquire from date only from non-blank input
-            reqfromdate = req.args['from'].strip()
+            reqfromdate = req.args.getfirst('from').strip()
             if reqfromdate:
                 try:
                     precisedate = user_time(req, parse_date, reqfromdate)
-                except TracError, e:
+                except TracError as e:
                     add_warning(req, e)
                 else:
                     fromdate = precisedate.astimezone(req.tz)
-            precision = req.args.get('precision', '')
+            precision = req.args.getfirst('precision', '')
             if precision.startswith('second'):
                 precision = timedelta(seconds=1)
             elif precision.startswith('minute'):
@@ -130,17 +131,12 @@ class TimelineModule(Component):
                                         fromdate.day, 23, 59, 59, 999999),
                                req.tz)
 
-        daysback = as_int(req.args.get('daysback'),
-                          90 if format == 'rss' else None)
-        if daysback is None:
-            daysback = as_int(req.session.get('timeline.daysback'), None)
-        if daysback is None:
-            daysback = self.default_daysback
-        daysback = max(0, daysback)
-        if self.max_daysback >= 0:
-            daysback = min(self.max_daysback, daysback)
+        pref = req.session.as_int('timeline.daysback', self.default_daysback)
+        default = 90 if format == 'rss' else pref
+        daysback = req.args.as_int('daysback', default,
+                                   min=1, max=self.max_daysback)
 
-        authors = req.args.get('authors')
+        authors = req.args.getfirst('authors')
         if authors is None and format != 'rss':
             authors = req.session.get('timeline.authors')
         authors = (authors or '').strip()
@@ -162,7 +158,7 @@ class TimelineModule(Component):
         filters = [f[0] for f in available_filters if f[0] in req.args]
         if not filters and format != 'rss':
             filters = [f[0] for f in available_filters
-                       if req.session.get('timeline.filter.' + f[0]) == '1']
+                            if req.session.as_int('timeline.filter.' + f[0])]
         if not filters:
             filters = [f[0] for f in available_filters if len(f) == 2 or f[2]]
 
@@ -195,12 +191,11 @@ class TimelineModule(Component):
             try:
                 for event in provider.get_timeline_events(req, start, stop,
                                                           filters) or []:
-                    # Check for 0.10 events
-                    author = (event[2 if len(event) < 6 else 4] or '').lower()
+                    author = (event[2] or '').lower()
                     if (not include or author in include) \
                             and author not in exclude:
                         events.append(self._event_data(provider, event))
-            except Exception, e:  # cope with a failure of that provider
+            except Exception as e:  # cope with a failure of that provider
                 self._provider_failure(e, req, provider, filters,
                                        [f[0] for f in available_filters])
 
@@ -212,7 +207,6 @@ class TimelineModule(Component):
         data['events'] = events
 
         if format == 'rss':
-            data['email_map'] = Chrome(self.env).get_email_map()
             rss_context = web_context(req, absurls=True)
             rss_context.set_hints(wiki_flavor='html', shorten_lines=False)
             data['context'] = rss_context
@@ -246,8 +240,8 @@ class TimelineModule(Component):
         # Navigation to the previous/next period of 'daysback' days
         previous_start = fromdate.replace(tzinfo=None) - \
                          timedelta(days=daysback + 1)
-        previous_start = format_date(to_datetime(previous_start, req.tz),
-                                     format='%Y-%m-%d', tzinfo=req.tz)
+        previous_start = format_date(previous_start, format='iso8601',
+                                     tzinfo=req.tz)
         add_link(req, 'prev', req.href.timeline(from_=previous_start,
                                                 authors=authors,
                                                 daysback=daysback),
@@ -256,7 +250,7 @@ class TimelineModule(Component):
             next_start = fromdate.replace(tzinfo=None) + \
                          timedelta(days=daysback + 1)
             next_start = format_date(to_datetime(next_start, req.tz),
-                                     format='%Y-%m-%d', tzinfo=req.tz)
+                                     format='iso8601', tzinfo=req.tz)
             add_link(req, 'next', req.href.timeline(from_=next_start,
                                                     authors=authors,
                                                     daysback=daysback),
@@ -281,27 +275,46 @@ class TimelineModule(Component):
     def post_process_request(self, req, template, data, content_type):
         if data:
             def pretty_dateinfo(date, format=None, dateonly=False):
-                absolute = user_time(req, format_datetime, date)
-                relative = pretty_timedelta(date)
+                if not date:
+                    return ''
+                if format == 'date':
+                    absolute = user_time(req, format_date, date)
+                else:
+                    absolute = user_time(req, format_datetime, date)
+                now = datetime_now(localtz)
+                relative = pretty_timedelta(date, now)
                 if not format:
                     format = req.session.get('dateinfo',
                                  Chrome(self.env).default_dateinfo_format)
-                if format == 'absolute':
+                if format == 'relative':
+                    if date > now:
+                        label = _("in %(relative)s", relative=relative) \
+                                if not dateonly else relative
+                        title = _("on %(date)s at %(time)s",
+                                  date=user_time(req, format_date, date),
+                                  time=user_time(req, format_time, date))
+                        return tag.span(label, title=title)
+                    else:
+                        label = _("%(relative)s ago", relative=relative) \
+                                if not dateonly else relative
+                        title = _("See timeline at %(absolutetime)s",
+                                  absolutetime=absolute)
+                else:
                     if dateonly:
                         label = absolute
                     elif req.lc_time == 'iso8601':
                         label = _("at %(iso8601)s", iso8601=absolute)
+                    elif format == 'date':
+                        label = _("on %(date)s", date=absolute)
                     else:
                         label = _("on %(date)s at %(time)s",
                                   date=user_time(req, format_date, date),
                                   time=user_time(req, format_time, date))
+                    if date > now:
+                        title = _("in %(relative)s", relative=relative)
+                        return tag.span(label, title=title)
                     title = _("See timeline %(relativetime)s ago",
                               relativetime=relative)
-                else:
-                    label = _("%(relativetime)s ago", relativetime=relative) \
-                            if not dateonly else relative
-                    title = _("See timeline at %(absolutetime)s",
-                              absolutetime=absolute)
                 return self.get_timeline_link(req, date, label,
                                               precision='second', title=title)
             def dateinfo(date):
@@ -332,7 +345,7 @@ class TimelineModule(Component):
                 dt = parse_date(path, utc, locale='iso8601', hint='iso8601')
                 return self.get_timeline_link(formatter.req, dt, label,
                                               precision, query, fragment)
-            except TracError, e:
+            except TracError as e:
                 return tag.a(label, title=to_unicode(e),
                              class_='timeline missing')
         yield 'timeline', link_resolver
@@ -353,19 +366,12 @@ class TimelineModule(Component):
     def _event_data(self, provider, event):
         """Compose the timeline event date from the event tuple and prepared
         provider methods"""
-        if len(event) == 6: # 0.10 events
-            kind, url, title, date, author, markup = event
-            data = {'url': url, 'title': title, 'description': markup}
-            render = lambda field, context: data.get(field)
-        else: # 0.11 events
-            if len(event) == 5: # with special provider
-                kind, date, author, data, provider = event
-            else:
-                kind, date, author, data = event
-            render = lambda field, context: \
-                    provider.render_timeline_event(context, field, event)
-        if not isinstance(date, datetime):
-            date = datetime.fromtimestamp(date, utc)
+        if len(event) == 5:  # with special provider
+            kind, date, author, data, provider = event
+        else:
+            kind, date, author, data = event
+        render = lambda field, context: \
+                 provider.render_timeline_event(context, field, event)
         dateuid = to_utimestamp(date)
         return {'kind': kind, 'author': author, 'date': date,
                 'dateuid': dateuid, 'render': render, 'event': event,
@@ -395,7 +401,7 @@ class TimelineModule(Component):
         raise TracError(tag(
             tag.p(tag_("Event provider %(name)s failed for filters "
                        "%(kinds)s: ",
-                       name=tag.tt(ep.__class__.__name__),
+                       name=tag.code(ep.__class__.__name__),
                        kinds=', '.join('"%s"' % ep_kinds[f] for f in
                                        current_filters & ep_filters)),
                   tag.strong(exception_to_unicode(exc)), class_='message'),
