@@ -285,6 +285,8 @@ else:
             if e.errno == errno.ENOENT:
                 with open(filename, 'ab'):
                     pass
+            else:
+                raise
 
 
 def create_zipinfo(filename, mtime=None, dir=False, executable=False, symlink=False,
@@ -1286,7 +1288,20 @@ def pathjoin(*args):
 
 def to_list(splittable, sep=','):
     """Split a string at `sep` and return a list without any empty items.
+
+    >>> to_list('1,2, 3,4 ')
+    ['1', '2', '3', '4']
+    >>> to_list('1;2; 3;4 ', sep=';')
+    ['1', '2', '3', '4']
+    >>> to_list('')
+    []
+    >>> to_list(None)
+    []
+    >>> to_list([])
+    []
     """
+    if not splittable:
+        return []
     split = [x.strip() for x in splittable.split(sep)]
     return [item for item in split if item]
 
