@@ -45,6 +45,7 @@ __all__ = ['wiki_to_html', 'wiki_to_oneliner', 'wiki_to_outline',
            'concat_path_query_fragment']
 
 
+
 def system_message(msg, text=None):
     return tag.div(tag.strong(msg), text and tag.pre(text),
                    class_="system-message")
@@ -160,7 +161,9 @@ class WikiProcessor(object):
                              'span': True, 'Span': True,
                              }.get(name)
 
-        self._sanitizer = TracHTMLSanitizer(formatter.wiki.safe_schemes)
+        self._sanitizer = TracHTMLSanitizer(
+            safe_schemes=formatter.wiki.safe_schemes,
+            safe_origins=formatter.wiki.safe_origins)
 
         self.processor = builtin_processors.get(name)
         if not self.processor:
@@ -804,7 +807,8 @@ class Formatter(object):
             return system_message(_("Macro %(name)s(%(args)s) failed",
                                     name=name, args=args), to_fragment(e))
         except Exception as e:
-            self.env.log.error('Macro %s(%s) failed:%s', name, args,
+            self.env.log.error("Macro %s(%s) failed for %s:%s", name,
+                               args, self.resource,
                                exception_to_unicode(e, traceback=True))
             return system_message(_("Error: Macro %(name)s(%(args)s) failed",
                                     name=name, args=args), to_fragment(e))
@@ -1195,11 +1199,15 @@ class Formatter(object):
     def _exec_processor(self, processor, text):
         try:
             return processor.process(text)
+        except ProcessorError as e:
+            return system_message(_("Processor %(name)s failed",
+                                    name=processor.name), to_fragment(e))
         except Exception as e:
-            self.env.log.error('Processor %s failed:%s', processor.name,
+            self.env.log.error("Processor %s failed for %s:%s",
+                               processor.name, self.resource,
                                exception_to_unicode(e, traceback=True))
             return system_message(_("Error: Processor %(name)s failed",
-                                    name=processor.name), to_unicode(e))
+                                    name=processor.name), to_fragment(e))
 
     # > quotes
 
