@@ -612,7 +612,7 @@ class ImageMacro(WikiMacroBase):
         if self._quoted_re.match(filespec):
             filespec = filespec.strip('\'"')
         # parse filespec argument to get realm and id if contained.
-        parts = [i.strip('''['"]''')
+        parts = [i.strip('\'"')
                  for i in self._split_filespec_re.split(filespec)[1::2]]
         realm = parts[0] if parts else None
         url = raw_url = desc = None
@@ -701,6 +701,9 @@ class ImageMacro(WikiMacroBase):
         if style:
             attr['style'] = '; '.join('%s:%s' % (k, escape(v))
                                       for k, v in style.iteritems())
+        if not WikiSystem(self.env).is_safe_origin(raw_url,
+                                                   formatter.context.req):
+            attr['crossorigin'] = 'anonymous'  # avoid password prompt
         result = tag.img(src=raw_url, **attr)
         if link is not None:
             result = tag.a(result, href=link or url,
