@@ -47,8 +47,6 @@ Warning:
   those properties...
 """
 
-from __future__ import with_statement
-
 import os.path
 import re
 import weakref
@@ -64,7 +62,7 @@ from trac.versioncontrol import Changeset, Node, Repository, \
 from trac.versioncontrol.cache import CachedRepository
 from trac.util import embedded_numbers
 from trac.util.concurrency import threading
-from trac.util.text import exception_to_unicode, to_unicode
+from trac.util.text import exception_to_unicode, to_unicode, to_utf8
 from trac.util.translation import _
 from trac.util.datefmt import from_utimestamp, to_datetime, utc
 from trac.web.href import Href
@@ -286,9 +284,9 @@ class SubversionConnector(Component):
         """End-of-Line character sequences when `svn:eol-style` property is
         `native`.
 
-        If `native` (the default), substitute with the native EOL marker on
-        the server. Otherwise, if `LF`, `CRLF` or `CR`, substitute with the
-        specified EOL marker.
+        If `native`, substitute with the native EOL marker on the server.
+        Otherwise, if `LF`, `CRLF` or `CR`, substitute with the specified
+        EOL marker.
 
         (''since 1.0.2'')""")
 
@@ -298,11 +296,11 @@ class SubversionConnector(Component):
         self._version = None
         try:
             _import_svn()
-            self.log.debug("Subversion bindings imported")
-        except ImportError, e:
+        except ImportError as e:
             self.error = e
             self.log.info('Failed to load Subversion bindings', exc_info=True)
         else:
+            self.log.debug("Subversion bindings imported")
             version = (core.SVN_VER_MAJOR, core.SVN_VER_MINOR,
                        core.SVN_VER_MICRO)
             self._version = '%d.%d.%d' % version + core.SVN_VER_TAG
@@ -314,7 +312,7 @@ class SubversionConnector(Component):
     # ISystemInfoProvider methods
 
     def get_system_info(self):
-        if self._version is not None:
+        if self._version:
             yield 'Subversion', self._version
 
     # IRepositoryConnector methods
@@ -368,7 +366,7 @@ class SubversionRepository(Repository):
 
         try:
             self.repos = repos.svn_repos_open(root_path_utf8, self.pool())
-        except core.SubversionException, e:
+        except core.SubversionException as e:
             raise InvalidRepository(
                 _("Couldn't open Subversion repository %(path)s: "
                   "%(svn_error)s", path=to_unicode(path_utf8),
@@ -770,7 +768,7 @@ class SubversionNode(Node):
         else:
             try:
                 self.root = fs.revision_root(self.fs_ptr, rev, pool)
-            except core.SubversionException, e:
+            except core.SubversionException as e:
                 raise NoSuchNode(path, rev, exception_to_unicode(e))
         node_type = fs.check_path(self.root, self._scoped_path_utf8, pool)
         if not node_type in _kindmap:
@@ -866,7 +864,7 @@ class SubversionNode(Node):
                 from svn import client
                 client.blame2(file_url_utf8, rev, start, rev, blame_receiver,
                               client.create_context(), self.pool())
-            except (core.SubversionException, AttributeError), e:
+            except (core.SubversionException, AttributeError) as e:
                 # svn thinks file is a binary or blame not supported
                 raise TracError(_('svn blame failed on %(path)s: %(error)s',
                                   path=self.path, error=to_unicode(e)))
@@ -1205,8 +1203,8 @@ class FileContentStream(object):
         else:
             return self._read_substitute(self.stream, n)
 
-    def _get_revprop(self, name):
-        return fs.revision_prop(self.fs_ptr, self.node.rev, name, self.pool())
+    def _get_revprop(self, name, rev):
+        return fs.revision_prop(self.fs_ptr, rev, name, self.pool())
 
     def _split_keywords(self, keywords):
         return filter(None, self.KEYWORD_SPLIT_RE.split(keywords or ''))
@@ -1218,12 +1216,13 @@ class FileContentStream(object):
 
         node = self.node
         mtime = to_datetime(node.last_modified, utc)
-        shortdate = mtime.strftime('%Y-%m-%d %H:%M:%SZ')
-        longdate = mtime.strftime('%Y-%m-%d %H:%M:%S +0000 (%a, %d %b %Y)')
+        shortdate = self._format_shortdate(mtime)
+        longdate = self._format_longdate(mtime)
         created_rev = unicode(node.created_rev)
         # Note that the `to_unicode` has a small probability to mess-up binary
         # properties, see #4321.
-        author = to_unicode(self._get_revprop(core.SVN_PROP_REVISION_AUTHOR))
+        author = to_unicode(self._get_revprop(core.SVN_PROP_REVISION_AUTHOR,
+                                              node.created_rev))
         path = node.path.lstrip('/')
         url = node.repos.get_path_url(path, node.rev) or path
         root_url = node.repos.get_path_url('', node.rev) or '/'
@@ -1254,7 +1253,7 @@ class FileContentStream(object):
                 values[name] = self.KEYWORD_EXPAND_RE.sub(expand, definition)
 
         if values:
-            return dict((key, value.encode('utf-8'))
+            return dict((key, to_utf8(value))
                         for key, value in values.iteritems())
         else:
             return None
@@ -1273,6 +1272,17 @@ class FileContentStream(object):
         else:
             return None
 
+    def _format_shortdate(self, mtime):
+        return mtime.strftime('%Y-%m-%d %H:%M:%SZ')
+
+    def _format_longdate(self, mtime):
+        text = mtime.strftime('%Y-%m-%d %H:%M:%S +0000 (%%(a)s, %d %%(b)s %Y)')
+        weekdays = ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun')
+        months = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
+                  'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec')
+        return text % {'a': weekdays[mtime.weekday()],
+                       'b': months[mtime.month - 1]}
+
     def _read_dumb(self, stream, n):
         return stream.read(n)
 
